
local gamera = require 'gamera';
------------------------------------------------------------
local floor = math.floor;
local mouse = { x = 0, y = 0 };
------------------------------------------------------------
local stuff = require 'stuff';
------------------------------------------------------------
local Player = require 'hitcar':subclass('Player')
function Player:move(dt)
	if lk("down","z") then
		self.EngSpd = self.RPow;
	elseif lk("up","x") then
		if self.EngSpd < 0 then self.EngSpd = 0; end
		self.EngSpd = self.EngSpd<self.FPow and self.EngSpd+10*dt or self.FPow;
	else
		self.EngSpd = 0;
	end
end

------------------------------------------------------------
local GAME = {

  	world = lp.newWorld( 0, 0, true),
  	trees = { },
  	map = {
  		w = 4000,
  		h = 4000
  	},

	wheel = {
		ang = 0,
		x = 246,
		y = 580,
		i = 'wheel',
		grab = false,
		tAng = 0,
		boo = true
	},

	hand = {
		i = 'hand',
		a = "open",
		color = {0, 50, 216},
		open = lg.newQuad(0,0,60,80,120,80),
		shut = lg.newQuad(60,0,60,80,120,80)
	},

	pedals = {
		i = 'pedals',
		ba = "b_up",
		ga = "g_up",
		b_up = lg.newQuad(0,0,39,60,140,60),
		b_down = lg.newQuad(70,0,39,60,140,60),
		g_up = lg.newQuad(40,0,30,60,140,60),
		g_down = lg.newQuad(110,0,30,60,140,60)
	},

	set = function( self )
		self.map.tw = floor(self.map.w/64)
		self.map.th = floor(self.map.h/64)
		self.cam = gamera.new(0,0,screen.w,screen.h)
		self.cam:setWorld(0,0,self.map.w,self.map.h)
		self.player = Player( self.map.w/2, self.map.h/2, self.world )
	end,
	update = function( self, dt )
		self.world:update(dt);
		mouse.x, mouse.y = love.mouse.getPosition();

		self.player:up( dt, self.wheel.ang );
		self.player:move( dt );

		if self.wheel.grab then
			local a = math.atan2(mouse.x-self.wheel.x,self.wheel.y-mouse.y) + math.pi/2;
			if self.wheel.boo then
				self.wheel.tAng = (self.wheel.ang - (self.wheel.ang - a)) - self.wheel.ang;
				self.wheel.boo = false;
			end
			self.wheel.ang = (a - self.wheel.tAng);
			if self.wheel.ang<-1.4 or self.wheel.ang>1.4 then
				self.wheel.ang = self.wheel.ang>1 and 1.35 or -1.35;
			end
		else
			self.wheel.boo = true;
			self.wheel.ang = (math.max(math.min(self.wheel.ang+5*dt,0),self.wheel.ang-5*dt));
		end

		-- love.graphics.setCaption("FPS: "..tostring(love.timer.getFPS( )));
		
		if love.mouse.isDown("l") then
			self.hand.a = "shut";
			local dr = dist(mouse.x,mouse.y,self.wheel.x,self.wheel.y);
			self.wheel.grab = (dr>20 and dr<110)
		else
			self.hand.a = "open";
			self.wheel.grab = false
		end
		if lk("down","z") then
			self.pedals.ba = "b_down";
			self.pedals.ga = "g_up";
		elseif lk("up","x") then
			self.pedals.ga = "g_down";
			self.pedals.ba = "b_up";
		else
			self.pedals.ga = "g_up";
			self.pedals.ba = "b_up";
		end
		self.cam:setPosition(self.player:getPos( ))
	end,
	draw = function( self )
		lg.setColor(255,255,255);
		self.cam:draw(function(l,t,w,h)
			-------------------------- DRAW MAP
			for i=l, (l+w+64), 64 do

				local x = floor(i/64)*64;
				for j=t, (t+h+64), 64 do

					local y = floor(j/64)*64;
					lg.draw(imgs[self.grass],x,y)
				end
			end
			--------------------------
			self.player:draw( )
			for i=1,3 do
				for n,s in ipairs(self.trees) do
					s["draw"..i]( s, self.cam )
				end
			end
		end)

		lg.setColor(255,255,255)
		lg.draw( imgs['hud'])

		lg.draw( imgs[self.wheel.i], self.wheel.x, self.wheel.y, self.wheel.ang, 1,1,100,100)

		lg.draw(imgs[self.pedals.i],self.pedals[self.pedals.ba],72,530 )
		lg.draw(imgs[self.pedals.i],self.pedals[self.pedals.ga],112,530 )

		lg.setColor(self.hand.color)
		lg.draw(imgs[self.hand.i],self.hand[self.hand.a],mouse.x,mouse.y,0,1,1,40,45)

	end,
	keypressed = function( self, k )
		-- self.player:key(k)
		if k == "escape" then love.event.quit() end
	end,
	mousepressed = function( self, x, y, b )
		local cx,cy = self.cam:toWorld( x, y )
		-- if b == "m" then table.insert( self.stuff, stuff.cone( self.world, cx, cy )) end
		if b == "r" then table.insert( self.trees, stuff.tree( self.world, cx, cy ))
			print(self.trees[1].name)
		end
	end,

	grass = 'grass',
	hud  = "hud",
	
};
return GAME;

