local tire = class("tire")
function tire:init( world, B, X, Y )
	self.b = lp.newBody(world, X, Y, "dynamic")
	self.s = lp.newRectangleShape(4,12)
	self.f = lp.newFixture(self.b, self.s, 1)
	self.j = lp.newRevoluteJoint( B, self.b, X, Y, false )
	-- self.b:setMass( 0 )
	self.b:resetMassData( )
	-- self.j:enableMotor(true)
	self.j:setMotorEnabled(true)
	self.j:setMaxMotorTorque(200)
end
function tire:draw(mode)
	lg.polygon(mode, self.b:getWorldPoints(self.s:getPoints()))
end
function tire:kill()
	self.b:destroy()
end
local function KillOrthoVel( body )
	local pn1 = vec.new( body:getWorldVector(1,0) )
	local vel = vec.new( body:getLinearVelocity( ) )
	local pn2 = pn1 * pn1:dot( vel )
	local pn3 = (-1*pn2)*body:getMass( )
	body:applyLinearImpulse( pn3.x, pn3.y );
end

local Car = class('Car')

function Car:init( X, Y, world )
	--------------------------------------------------------------------------
	self.SteerSpd = 60;
	self.FPow = 150;
	self.RPow = -30;
	self.EngSpd = 0;
	self.MaxSteerAng = 0.4;
	self.SteerAng = 0;
	self.obj = {};
	self.obj.body = lp.newBody( world, 0 , 0, "dynamic")
	self.obj.body:setLinearDamping(1)
	self.obj.body:setAngularDamping(1)
	-- self.obj.shape = lp.newRectangleShape(0,0,40,70 )
	self.obj.shape = lp.newPolygonShape(-20,-31,-16,-35,16,-35,20,-31,20,31,16,35,-16,35,-20,31)
	self.obj.fixture = lp.newFixture(self.obj.body, self.obj.shape, 1)
	self.obj.body:setMass( 0 )
	-- self.obj.body:resetMassData( )
	self.obj.body:setPosition(X,Y)
	---Wheels
	self.obj.lfw = tire( world, self.obj.body,X-12, Y-20 )
	self.obj.rfw = tire( world, self.obj.body,X+12, Y-20 )
	self.obj.lrw = tire( world, self.obj.body,X-12, Y+20 )
	self.obj.rrw = tire( world, self.obj.body,X+12, Y+20 )

	self.obj.body:setAwake( true );
	--------------------------------------------------------------------------
	self.a = 0;
	self.t = 0;
	self.r = 0;
end

function Car:getPos( )
	return self.obj.body:getPosition()
end

function Car:up(dt, wheelAng)

	self.SteerAng = wheelAng/3;
	KillOrthoVel(self.obj.lfw.b)
	KillOrthoVel(self.obj.rfw.b)
	KillOrthoVel(self.obj.lrw.b)
	KillOrthoVel(self.obj.rrw.b)

	local ang=self.obj.lfw.b:getAngle()-math.rad(90)
	self.obj.lfw.b:applyForce(math.cos(ang)*self.EngSpd,math.sin(ang)*self.EngSpd)

	ang=self.obj.rfw.b:getAngle()-math.rad(90)
	self.obj.rfw.b:applyForce(math.cos(ang)*self.EngSpd,math.sin(ang)*self.EngSpd)

	self.obj.lrw.b:setAngle(self.obj.body:getAngle())
	self.obj.rrw.b:setAngle(self.obj.body:getAngle())

	local mspeed = self.SteerAng - self.obj.lfw.j:getJointAngle();
	self.obj.lfw.j:setMotorSpeed(mspeed * self.SteerSpd);

	mspeed = self.SteerAng - self.obj.rfw.j:getJointAngle();
	self.obj.rfw.j:setMotorSpeed(mspeed * self.SteerSpd);
end

function Car:draw()

	lg.setColor(255,255,255,80);

	self.obj.lfw:draw("fill")
	self.obj.rfw:draw("fill")
	self.obj.lrw:draw("fill")
	self.obj.rrw:draw("fill")

	lg.polygon("line", self.obj.body:getWorldPoints(self.obj.shape:getPoints()))
end

function Car:kill()
	self.obj.lfw:kill()
	self.obj.rfw:kill()
	self.obj.lrw:kill()
	self.obj.rrw:kill()
	self.b:destroy()
end

return Car;
